#!/usr/local/bin/python

"""
Copyright (c) 2004, Jan Stocker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Jan Stocker nor the names of its contributors may
      be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
"""

from Tkinter import *
import os
import tkFileDialog
import tkMessageBox

import ConfigParser

from thudpdfreport import *

flagIsWin32 = True
flagHasWinAPI = True
try:
    winver = sys.winver
    try:
        import win32api
    except:
        flagHasWinAPI = False
except:
    flagIsWin32 = False
    flagHasWinAPI = False



class WndThudToolsInfo(object):
    def __init__(self, app):
        self.app = app
        self.frame = None

    def createWindow(self, x, y, width, height):
        self.frame = Frame(self.app, width=width, height=height, bg=self.app.colorWidgetBackground)
        self.frame.place(x=x, y=y, anchor=NW)
        
        posX = 0
        posY = 0

        self.labelThudfile = Label(self.frame,
			text="ThudBoard Tools v0.2 - 2004-10-19\n(c) 2004 Jan Stocker\n\nSend comments and bug reports to:\n\nJan.Stocker@t-online.de",
			font=self.app.font, bg=self.app.colorWidgetBackground, foreground=self.app.colorText, wraplength=width)
        self.labelThudfile.place(x=posX, y=posY, anchor=NW)
    
    def destroyWindow(self):
        self.frame.destroy()



class WndThudToolsMerge(object):
    def __init__(self, app):
        self.app = app
        self.frame = None

    def createWindow(self, x, y, width, height):
        self.frame = Frame(self.app, width=width, height=height, bg=self.app.colorWidgetBackground)
        self.frame.place(x=x, y=y, anchor=NW)
        
        posX = 0
        posY = 0

        self.labelThudfile = Label(self.frame, text="Coming soon: Merging of thud comments...", font=self.app.font, bg=self.app.colorWidgetBackground, foreground=self.app.colorText)
        self.labelThudfile.place(x=posX, y=posY, anchor=NW)
    
    def destroyWindow(self):
        self.frame.destroy()


class ConfigThudToolsPDF(object):
    flagOpenPDF = True
    thudfile = ""
    pdffile = ""
    namePDFViewer = ""
    nMode = 1
    

    def load(self, filename, section="ThudToolsPDF"):
        config = ConfigParser.ConfigParser()
        config.read(filename)
        try:
            self.flagCoordsAround = config.getboolean(section, 'flagOpenPDF')
            self.thudfile = config.get(section, 'filenameThud')
            self.pdffile  = config.get(section, 'filenamePDF')
            self.namePDFViewer  = config.get(section, 'namePDFViewer')
            self.nMode = config.getint(section, 'nMode')
        except:
            pass 

    def save(self, filename, section="ThudToolsPDF"):
        config = ConfigParser.ConfigParser()
        config.read(filename)

        if not config.has_section(section):
            config.add_section(section)
        try:
            config.set(section, 'flagOpenPDF', self.flagOpenPDF) 
            config.set(section, 'filenameThud', self.thudfile) 
            config.set(section, 'filenamePDF', self.pdffile) 
            config.set(section, 'namePDFViewer', self.namePDFViewer) 
            config.set(section, 'nMode', self.nMode) 
            fp = open(filename, 'w+b')
            config.write(fp)
            fp.close()
        except:
            showwarning('Save options failed', \
                        'Saving the configuration failed. The file "'+filename+'" or folder may be read-only or corrupt.')

class WndThudToolsPDF(object):
    def __init__(self, app):
        self.app = app
        self.frame = None
        self.PDFReport = ThudPDFReport()
        self.cfg = ConfigThudToolsPDF()
        self.imgPrinter    = PhotoImage(file=os.sep.join(['img', 'printer.gif']))

    def createWindow(self, x, y, width, height):
        self.cfg.load(self.app.cfgfile)
        self.PDFReport.cfg.load(self.app.cfgfile)

        self.frame = Frame(self.app, width=width, height=height, bg=self.app.colorWidgetBackground)
        self.frame.place(x=x, y=y, anchor=NW)
        
        posX = 0
        posY = 0

        self.labelThudfile = Label(self.frame, text="Savegame:", font=self.app.font, bg=self.app.colorWidgetBackground, foreground=self.app.colorText)
        self.labelThudfile.place(x=posX, y=posY, anchor=NW)
        self.editThudfile = Entry(self.frame, width=23, font=self.app.font, foreground=self.app.colorText)
        self.editThudfile.place(x=posX+84, y=posY, anchor=NW)
        self.buttonBrowseThudfile = Button(self.frame, text='...', command=self.browseThudfile, font=self.app.font, bg=self.app.colorWidgetBackground, activebackground=self.app.colorWidgetBackground, foreground=self.app.colorText, activeforeground=self.app.colorText)
        self.buttonBrowseThudfile.place(x=width, y=posY, height=18, anchor=NE)
        posY = posY+30

        self.labelPDFFile = Label(self.frame, text="PDF file:", font=self.app.font, bg=self.app.colorWidgetBackground, foreground=self.app.colorText)
        self.labelPDFFile.place(x=posX, y=posY, anchor=NW)
        self.editPDFFile = Entry(self.frame, width=23, font=self.app.font, foreground=self.app.colorText)
        self.editPDFFile.place(x=posX+84, y=posY, anchor=NW)
        self.buttonBrowsePDFFile = Button(self.frame, text='...', command=self.browsePDFFile, font=self.app.font, bg=self.app.colorWidgetBackground, activebackground=self.app.colorWidgetBackground, foreground=self.app.colorText, activeforeground=self.app.colorText)
        self.buttonBrowsePDFFile.place(x=width, y=posY, height=18, anchor=NE)
        posY = posY+30

        self.mode = IntVar()
        self.radioModeOne = Radiobutton(self.frame, text="Create one move per line", variable=self.mode, font=self.app.font, bg=self.app.colorWidgetBackground, activebackground=self.app.colorWidgetBackground, foreground=self.app.colorText, activeforeground=self.app.colorText, highlightbackground=self.app.colorWidgetBackground, value=1)
        self.radioModeOne.place(x=posX, y=posY, anchor=NW)
        posY = posY+30

        self.radioModeTwo = Radiobutton(self.frame, text="Create two moves per line", variable=self.mode, font=self.app.font, bg=self.app.colorWidgetBackground, activebackground=self.app.colorWidgetBackground, foreground=self.app.colorText, activeforeground=self.app.colorText, highlightbackground=self.app.colorWidgetBackground, value=2)
        self.radioModeTwo.place(x=posX, y=posY, anchor=NW)
        posY = posY+30

        self.flagOpenPDF = IntVar()
        self.checkboxOpenPDF = Checkbutton(self.frame, text="Open PDF after creation", variable=self.flagOpenPDF, font=self.app.font, bg=self.app.colorWidgetBackground, activebackground=self.app.colorWidgetBackground, foreground=self.app.colorText, activeforeground=self.app.colorText, highlightbackground=self.app.colorWidgetBackground)
        self.checkboxOpenPDF.place(x=posX, y=posY, anchor=NW)
        posY = posY+30

        self.labelPDFViewer = Label(self.frame, text="PDF Viewer:", font=self.app.font, bg=self.app.colorWidgetBackground, foreground=self.app.colorText)
        self.editPDFViewer = Entry(self.frame, width=23, font=self.app.font, foreground=self.app.colorText)
        self.buttonBrowsePDFViewer = Button(self.frame, text='...', command=self.browsePDFViewer, font=self.app.font, bg=self.app.colorWidgetBackground, activebackground=self.app.colorWidgetBackground, foreground=self.app.colorText, activeforeground=self.app.colorText)
        if not flagHasWinAPI:
            self.labelPDFViewer.place(x=posX, y=posY, anchor=NW)
            self.editPDFViewer.place(x=posX+84, y=posY, anchor=NW)
            self.buttonBrowsePDFViewer.place(x=width, y=posY, height=18, anchor=NE)
            posY = posY+30

        self.buttonOptions = Button(self.frame, text="Options...", command=self.editOptions, font=self.app.font, bg=self.app.colorWidgetBackground, activebackground=self.app.colorWidgetBackground, foreground=self.app.colorText, activeforeground=self.app.colorText)
        self.buttonOptions.place(x=posX, y=posY, anchor=NW)
        posY = posY+30

        self.buttonOptions = Button(self.frame, text=">>> Create PDF <<<", command=self.createPDFFile, font=self.app.font, bg=self.app.colorWidgetBackground, activebackground=self.app.colorWidgetBackground, image=self.imgPrinter, foreground=self.app.colorText, activeforeground=self.app.colorText)
        self.buttonOptions.place(x=posX+104, y=posY, anchor=NW)
        posY = posY+30
        
        self.updateData(False)
    
    def destroyWindow(self):
        self.updateData(True)
        self.cfg.save(self.app.cfgfile)
        self.PDFReport.cfg.save(self.app.cfgfile)
        self.frame.destroy()
    
    def updateData(self, flagDirection):
        if not flagDirection:
            self.editThudfile.delete("0", END)
            self.editThudfile.insert(END, self.cfg.thudfile)
            
            self.editPDFFile.delete("0", END)
            self.editPDFFile.insert(END, self.cfg.pdffile)

            if self.cfg.nMode == 2:
                self.radioModeTwo.select()
            else:
                self.radioModeOne.select()

            if self.cfg.flagOpenPDF:
                self.checkboxOpenPDF.select()
            else:
                self.checkboxOpenPDF.deselect()
                
            if not flagHasWinAPI:
                self.editPDFViewer.delete("0", END)
                self.editPDFViewer.insert(END, self.cfg.namePDFViewer)
        else:
            self.cfg.thudfile = self.editThudfile.get()
            self.cfg.pdffile  = self.editPDFFile.get()
            self.cfg.nMode    = self.mode.get()
            self.cfg.flagOpenPDF = self.flagOpenPDF.get()
            if not flagHasWinAPI:
                self.cfg.namePDFViewer = self.editPDFViewer.get()

    def editOptions(self):
        self.PDFReport.cfg.showEditDialog(self.frame)
    
    def browseThudfile(self):
        ft = [("Thud Battles", ".thud"),
              ("All files", "*")]
        filename = tkFileDialog.askopenfilename(initialdir=self.editPDFFile.get(), defaultextension=".thud",filetypes=ft)
        if filename:
            self.editThudfile.delete("0", END)
            self.editThudfile.insert(END, filename)
            
            self.editPDFFile.delete("0", END)
            self.editPDFFile.insert(END, filename+".pdf")
        
    def browsePDFFile(self):
        ft = [("PDF files", ".pdf"),
              ("All files", "*")]
        filename = tkFileDialog.askopenfilename(initialdir=self.editPDFFile.get(), defaultextension=".pdf",filetypes=ft)
        if filename:
            self.editPDFFile.delete("0", END)
            self.editPDFFile.insert(END, filename)

    def browsePDFViewer(self):
        ft = []
        if flagIsWin32:
            ft = [("Executable files", ".exe"),
                  ("All files", "*")]
        else:
            ft = [("All files", "*")]
        filename = tkFileDialog.askopenfilename(initialdir=self.editPDFViewer.get(),filetypes=ft)
        if filename:
            self.editPDFViewer.delete("0", END)
            self.editPDFViewer.insert(END, filename)

    def createPDFFile(self):
        self.updateData(True)
        self.PDFReport.createPDF(self.cfg.thudfile, self.cfg.pdffile, self.cfg.nMode)
        if self.cfg.flagOpenPDF:
            try:
                if flagIsWin32:
                    if flagHasWinAPI:
                        win32api.ShellExecute (0, "open", self.cfg.pdffile, None, ".", 0)
                    else:
                        namePDFViewer = self.editPDFViewer.get()
                        os.spawnl(os.P_NOWAIT, self.cfg.namePDFViewer, self.cfg.namePDFViewer, self.cfg.pdffile)
                else:
                    namePDFViewer = self.editPDFViewer.get()
                    os.spawnlp(os.P_NOWAIT, self.cfg.namePDFViewer, self.cfg.namePDFViewer, self.cfg.pdffile)
            except:
                tkMessageBox.showwarning(
                        "Operation failed",
                        "The PDF viewer could not be started!"
                        )




class App(Tk):
    def __init__(self):
        cfgdir = os.sep.join([os.path.expanduser('~'),'.thudtools'])
        if not os.path.exists(cfgdir):
            try:
                os.makedirs(cfgdir)
            except:
                pass
        self.cfgfile = os.sep.join([cfgdir, 'thudtools.cfg'])

        # setup app-window
        self.startup = True
        Tk.__init__(self)
        self.wm_title("ThudBoard - Tools")
        self.configure(background='#000000')
        self.wm_minsize(440, 500)
        self.wm_maxsize(440, 500)
        self.wm_geometry('440x500+0+0')
        self.wm_resizable(False, False)
        self.focus_force()

        self.protocol("WM_DELETE_WINDOW", self.destroyWindow)

        pointsize = self.calc_pointsize(10.7)
        try: 
            winver = sys.winver
            self.font = ("microsoft sans serif", pointsize)
        except:
            self.font = ("helvetica", pointsize)

        self.imgBackground = PhotoImage(file=os.sep.join(['img', 'bcktile.gif']))
        self.imgHead       = PhotoImage(file=os.sep.join(['img', 'head.gif']))
        self.imgWidgetback = PhotoImage(file=os.sep.join(['img', 'widgetback.gif']))

        self.colorWidgetBackground = '#e3d7a6'
        self.colorWidgetBackgroundDark = '#c3a786'
        self.colorText = '#000000'

        self.background = Canvas(self, width=440, height=500, borderwidth=0, bg=self.colorWidgetBackground, highlightthickness=0)
        self.background.pack()

        for x in range(3):
            for y in range(5):
                self.background.create_image(x*200, y*130, 
                         image=self.imgBackground, 
                        anchor=NW)

        self.background.create_image(60, 0, 
                image=self.imgHead, 
                anchor=NW)
        self.background.create_image(64,105,
                image=self.imgWidgetback,
                anchor=NW)

        self.buttonShowPDF = Button(self, text='PDF', font=self.font, bg=self.colorWidgetBackgroundDark, activebackground=self.colorWidgetBackgroundDark, foreground=self.colorText, activeforeground=self.colorText, borderwidth=0, command=self.showPDF)
        self.buttonShowPDF.place(x=76, y=120, anchor=NW)

        self.buttonShowPDF = Button(self, text='Merge', font=self.font, bg=self.colorWidgetBackgroundDark, activebackground=self.colorWidgetBackgroundDark, foreground=self.colorText, activeforeground=self.colorText, borderwidth=0, command=self.showMerge)
        self.buttonShowPDF.place(x=130, y=120, anchor=NW)

        self.buttonShowInfo = Button(self, text='Info', font=self.font, bg=self.colorWidgetBackgroundDark, activebackground=self.colorWidgetBackgroundDark, foreground=self.colorText, activeforeground=self.colorText, borderwidth=0, command=self.showInfo)
        self.buttonShowInfo.place(x=360, y=120, anchor=NE)

        self.wndTab = False
        self.wndPdf = WndThudToolsPDF(self)
        self.wndMerge = WndThudToolsMerge(self)
        self.wndInfo = WndThudToolsInfo(self)
        self.showPDF()

    def destroyWindow(self, event = None):
        self.wndTab.destroyWindow()
        self.destroy()

    def showPDF(self):
        if self.wndTab:
            self.wndTab.destroyWindow()

        self.wndTab = self.wndPdf
        self.wndTab.createWindow(76,150,276,290)
        
    def showMerge(self):
        if self.wndTab:
            self.wndTab.destroyWindow()

        self.wndTab = self.wndMerge
        self.wndTab.createWindow(76,150,276,290)

    def showInfo(self):
        if self.wndTab:
            self.wndTab.destroyWindow()

        self.wndTab = self.wndInfo
        self.wndTab.createWindow(76,150,276,290)


    
    def calc_pointsize(self, pixelapprox):
        """calculate relative pointsize to screen layout"""
        """(c) Marc Boeren / thud.py"""
        try:
            dpiy = self.winfo_screenheight()/(self.winfo_screenmmheight()/25.4)
        except:
            return 8
        pointsize = 72.0*pixelapprox/dpiy
        return int(pointsize)



def main():
    app = App()
    app.mainloop()

if __name__=='__main__':
    main()
